--
--  AutoBar
--
--  Config functions
--
--  Maintained by Azethoth / Toadkiller of Proudmoore.  Original author Saien of Hyjal
--  http://www.curse-gaming.com/en/wow/addons-4430-1-autobar-toadkiller.html
--

AutoBarConfig = {};
local dewdrop = DewdropLib:GetInstance("1.0");
AutoBarConfig.editPlayer = "_SHARED1";
AutoBarConfig.title = GetAddOnMetadata("AutoBar", "Title") .. " ("..GetAddOnMetadata("AutoBar", "Version")..")";


function AutoBarConfig_Toggle(msg)
	if (AutoBarConfigFrame:IsVisible()) then
		AutoBarConfigFrame:Hide();
	else
		AutoBarConfigFrame:Show();
	end
end


function AutoBarConfig.OnLoad()
	SLASH_AUTOBAR1 = "/autobar";
	SLASH_AUTOBAR2 = "/atb";
	SlashCmdList["AUTOBAR"] = function (msg)
	    -- no damn keyboard commands, just toggle config dialog
		AutoBarConfig_Toggle(msg);
	end
end


function AutoBarConfig.OnShow()
	AutoBarConfig.editPlayer = AutoBarProfile:GetEditPlayer();
	AutoBarConfig:SlotsInitialize();
	AutoBarConfig:SetSharedInitialize();
	AutoBarConfigSliderValueChanged();
	AutoBarConfig:SlotsViewInitialize();
	AutoBarConfig:CheckBoxInitialize();
	AutoBarConfig:DockToInitialize();
	AutoBarConfig:RevertInitialize();
end


-- Clone current state in AutoBarConfig.revert
function AutoBarConfig:RevertInitialize()
	if (not AutoBarConfig.revert) then
		AutoBarConfig.revert = {};
		AutoBarConfig.revert.profiles = {};
		AutoBarConfig.revert.profiles = AutoBarProfile.CloneProfiles();
		AutoBarConfig.revert.profile = AutoBarProfile.CloneUserProfile();
	end
end


-- Revert to the previous state stashed in AutoBarConfig.revert
function AutoBarConfig:Revert()
	if (AutoBarConfig.revert.profiles) then
		AutoBarProfile.RevertProfiles(AutoBarConfig.revert.profiles);
	end
	AutoBar_Config[AutoBar.currentPlayer].profile = AutoBarConfig.revert.profile;
	AutoBarProfile:ProfileChanged();
	AutoBarProfile:ProfileEditingChanged();
	AutoBar_SetupVisual();
	AutoBarConfig.revert = nil;
	AutoBarConfig.OnShow();
end


-- Delete previous state stashed in AutoBarConfig.revert
function AutoBarConfig:Done()
	AutoBarConfig.revert = nil;
	AutoBarConfigFrame:Hide();
end


function AutoBarConfig:SlotsViewInitialize()
	AutoBarConfig:SlotsUpdate("AutoBarConfigFrameSlotsView_Button", AutoBar.buttons);
end


function AutoBarConfig:SlotsInitialize()
	local profile = AutoBar_Config[AutoBar.currentPlayer].profile;
	local layerName = AUTOBAR_CONFIG_CHARACTER;
	if (profile.edit == 2) then
		layerName = AUTOBAR_CONFIG_SHARED;
	elseif (profile.edit == 3) then
		layerName = AUTOBAR_CONFIG_CLASS;
	elseif (profile.edit == 4) then
		layerName = AUTOBAR_CONFIG_BASIC;
	end
	AutoBarConfigFrameSlotsTitleText:SetText(layerName..AUTOBAR_CONFIG_SLOTEDITTEXT);
	AutoBarConfig:SlotsUpdate("AutoBarConfigFrameSlots_Button", AutoBar_Config[AutoBarConfig.editPlayer].buttons);
end


function AutoBarConfig:SlotsUpdate(buttonBaseName, buttons)
	local index, button, hotkey;

	for index = 1, AUTOBAR_MAXBUTTONS, 1 do
		button = getglobal(buttonBaseName..index);
		hotkey = getglobal(buttonBaseName..index.."HotKey");
		count  = getglobal(buttonBaseName..index.."Count");
		icon   = getglobal(buttonBaseName..index.."Icon");
		hotkey:SetText("#"..index);
		if (buttons[index]) then
			icon:SetTexture(AutoBar_GetTexture(buttons[index]));
			count:SetText("");
		else
			count:SetText(AUTOBAR_CONFIG_EMPTY);
			icon:SetTexture("");
		end
	end
end


function AutoBarConfig:ConfigSetTooltip(message)
	if (message and not AutoBar_Config[AutoBar.currentPlayer].profile.hideConfigTooltips) then
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		GameTooltip:SetText(message);
	end
end


function AutoBarConfig:ButtonSetTooltip()
	local buttonInfo;
	local preamble, extended, editable;
	if (this.itemid) then
		local name,itemid = GetItemInfo(this.itemid);
		if (name and itemid) then
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink(itemid);
		else
			local tmp = "item:"..this.itemid..":0:0:0";
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink("item:"..this.itemid..":0:0:0");
		end
		return;
	end
	if (this.category) then
		buttonInfo = this.category;
		extended=true;
	elseif (string.find(this:GetName(), "^AutoBarConfigFrameSlots_Button")) then
		buttonInfo = AutoBar_Config[AutoBarConfig.editPlayer].buttons[this:GetID()];
		extended = true;
		preamble = true;
		editable = true;
	elseif (string.find(this:GetName(), "^AutoBarConfigFrameSlotsView_Button")) then
		buttonInfo = AutoBar.buttons[this:GetID()];
		extended = true;
		preamble = true;
		editable = false;
	elseif (string.find(this:GetName(), "^AutoBarConfigSlotFrame_Button")) then
		extended=true;
		editable = AutoBarConfigSlot.editable;
		local tmp = AutoBarConfigSlot.slots[AutoBarConfigSlot.slotsIndex];
		if (type(tmp) == "table") then
			buttonInfo = tmp[this:GetID()]
		else
			buttonInfo = tmp;
		end
	end
	local message = "";
	if (buttonInfo == "EMPTY") then
	elseif (type(buttonInfo) == "table") then
		if (preamble) then message = AUTOBAR_TOOLTIP9; end
		local index,cat;
		for index, cat in pairs(buttonInfo) do
			if (type(cat) == "string" and AutoBar_Category_Info[cat]) then
				message = message.."\n"..AutoBar_Category_Info[cat].description;
			elseif (type(cat) == "number") then
				local name = GetItemInfo(cat);
				if (name) then
					message = message.."\n"..name..AUTOBAR_TOOLTIP10;
				else
					message = message..AUTOBAR_TOOLTIP11;
				end
--			else
--				message = message.."\n"..cat..AUTOBAR_TOOLTIP12;
			end
		end
	elseif (type(buttonInfo) == "string" and AutoBar_Category_Info[buttonInfo]) then
		if (preamble) then message = AUTOBAR_TOOLTIP13; end
		message = message..AutoBar_Category_Info[buttonInfo].description;
		if (extended) then
			message = message.."\n";
			if (AutoBar_Category_Info[buttonInfo].notusable) then
				message = message..AUTOBAR_TOOLTIP14;
			end
			if (AutoBar_Category_Info[buttonInfo].targetted) then
				if (AutoBar_Category_Info[buttonInfo].targetted == "WEAPON") then
					message = message..AUTOBAR_TOOLTIP15;
				else
					message = message..AUTOBAR_TOOLTIP16;
				end
			end
			if (AutoBar_Category_Info[buttonInfo].noncombat) then
				message = message..AUTOBAR_TOOLTIP17;
			end
			if (AutoBar_Category_Info[buttonInfo].combatonly) then
				message = message..AUTOBAR_TOOLTIP18;
			end
			if (AutoBar_Category_Info[buttonInfo].location) then
				message = message..AUTOBAR_TOOLTIP19..AutoBar_Category_Info[buttonInfo].location..".";
			end
			if (AutoBar_Category_Info[buttonInfo].limit) then
				message = message..AUTOBAR_TOOLTIP20;
				if (AutoBar_Category_Info[buttonInfo].limit.downhp) then
					message = message..AUTOBAR_TOOLTIP21;
					if (AutoBar_Category_Info[buttonInfo].limit.downmana) then
						message = message..", ";
					end
				end
				if (AutoBar_Category_Info[buttonInfo].limit.downmana) then
					message = message..AUTOBAR_TOOLTIP22;
				end
			end

		end
	elseif (type(buttonInfo) == "string" and not AutoBar_Category_Info[buttonInfo]) then
		if (preamble) then message = AUTOBAR_TOOLTIP23; end
		message = message..buttonInfo..AUTOBAR_TOOLTIP12;
	elseif (type(buttonInfo) == "number") then
		if (preamble) then message = AUTOBAR_TOOLTIP23; end
		local name = GetItemInfo(buttonInfo);
		if (name) then
			message = message..name..AUTOBAR_TOOLTIP10;
		else
			message = message..AUTOBAR_TOOLTIP11;
		end
	end

--	if (editable) then
--		message = message .. "\n" .. AUTOBAR_CONFIG_CLICK_EDIT;
--	end
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	GameTooltip:SetText(message);
end


-- Hook PickupContainerItem so we know what bag & slot is being dragged from
AutoBarConfig.PickupContainerItem = PickupContainerItem;
local dragBag = nil;
local dragSlot = nil;

function PickupContainerItem(bag, slot, ...)
	local returnTab = { AutoBarConfig.PickupContainerItem(bag, slot, unpack(arg)) };

	if (CursorHasItem()) then
		dragBag = bag;
		dragSlot = slot;
	end

	return unpack(returnTab);
end


function AutoBarConfig:GetDragBagSlot()
	return dragBag, dragSlot;
end


function AutoBarConfig:ViewButtonOnClick(mousebutton)
	ResetCursor();
	AutoBar.dragging = nil;
	return AutoBarConfigSlot:View(this:GetID(), AutoBar);
end


function AutoBarConfig:ButtonOnClick(mousebutton)
	ResetCursor();
	AutoBar.dragging = nil;
	return AutoBarConfigSlot:Edit(this:GetID(), AutoBar_Config[AutoBarConfig.editPlayer]);
end


function AutoBarConfig.ViewButtonOnDragStart()
	local fromIndex = this:GetID();
	AutoBar.SetDraggingIndex("AutoBarConfigFrameView", fromIndex);
	SetCursor("BUY_CURSOR");
end


function AutoBarConfig.ButtonOnDragStart()
	local fromIndex = this:GetID();
	AutoBar.SetDraggingIndex("AutoBarConfigFrame", fromIndex);
	SetCursor("BUY_CURSOR");
end


function AutoBarConfig.ButtonOnReceiveDrag()
	local toIndex = this:GetID();
	local fromIndex = AutoBar.GetDraggingIndex("AutoBarConfigFrame");
	local fromViewIndex = AutoBar.GetDraggingIndex("AutoBarConfigFrameView");
	if (fromIndex and fromIndex ~= toIndex) then
		local fromButton = AutoBar_Config[AutoBarConfig.editPlayer].buttons[fromIndex];
		AutoBar_Config[AutoBarConfig.editPlayer].buttons[fromIndex] = AutoBar_Config[AutoBarConfig.editPlayer].buttons[toIndex];
		AutoBar_Config[AutoBarConfig.editPlayer].buttons[toIndex] = fromButton;
	elseif (fromViewIndex) then
		AutoBar_Config[AutoBarConfig.editPlayer].buttons[toIndex] = AutoBar.buttons[fromViewIndex];
	end
	AutoBarProfile:ButtonsChanged();
	AutoBar.dragging = nil;
	ResetCursor();
end

function AutoBarConfig.OnEnter()
	if (AutoBar.dragging) then
		SetCursor("BUY_CURSOR");
	end
end


--
-- Slider handling
--

AutoBarConfigSliders =
{
	rows =
	{
		["name"] = "AutoBarConfigFrameBarRows",
		["text"] = AUTOBAR_CONFIG_ROW.." - ",
		["default"] = 1,
	},
	columns =
	{
		["name"] = "AutoBarConfigFrameBarColumns",
		["text"] = AUTOBAR_CONFIG_COLUMN.." - ",
		["default"] = AUTOBAR_MAXBUTTONS,
	},
	gapping =
	{
		["name"] = "AutoBarConfigFrameBarGapping",
		["text"] = AUTOBAR_CONFIG_GAPPING.." - ",
		["default"] = 3,
	},
	alpha =
	{
		["name"] = "AutoBarConfigFrameButtonsAlpha",
		["text"] = AUTOBAR_CONFIG_ALPHA.." - ",
		["default"] = 100,
	},
	buttonWidth =
	{
		["name"] = "AutoBarConfigFrameBarButtonWidth",
		["text"] = AUTOBAR_CONFIG_BUTTONWIDTH.." - ",
		["default"] = 36,
	},
	buttonHeight =
	{
		["name"] = "AutoBarConfigFrameBarButtonHeight",
		["text"] = AUTOBAR_CONFIG_BUTTONHEIGHT.." - ",
		["default"] = 36,
	},
	dockShiftX =
	{
		["name"] = "AutoBarConfigFrameBarDockShiftX",
		["text"] = AUTOBAR_CONFIG_DOCKSHIFTX.." - ",
		["default"] = 0,
	},
	dockShiftY =
	{
		["name"] = "AutoBarConfigFrameBarDockShiftY",
		["text"] = AUTOBAR_CONFIG_DOCKSHIFTY.." - ",
		["default"] = 0,
	},
}


function AutoBarConfig:SliderGetValue(sliderName, calledFromSlider)
	local value = nil;
	if (calledFromSlider) then
		value = getglobal(AutoBarConfigSliders[sliderName]["name"]):GetValue();
	end
	if ((not value or value == 0) and AutoBar_Config[AutoBarConfig.editPlayer].display[sliderName]) then
		value = AutoBar_Config[AutoBarConfig.editPlayer].display[sliderName];
	elseif (not value or value == 0) then
		value = AutoBarConfigSliders[sliderName]["default"];
	end
	return value;
end


function AutoBarConfig:SliderSetValue(sliderName, value, textValue)
	if (not textValue) then
		textValue = tostring(value);
	end
--DEFAULT_CHAT_FRAME:AddMessage("AutoBarConfig:SliderSetValue " ..sliderName);
	getglobal(AutoBarConfigSliders[sliderName]["name"] .. "Text"):SetText(AutoBarConfigSliders[sliderName]["text"]..textValue);
	getglobal(AutoBarConfigSliders[sliderName]["name"]):SetValue(value);
end


function AutoBarConfigSliderValueChanged(calledFromSlider)
	if (not AutoBar_Config[AutoBarConfig.editPlayer].display) then
		AutoBar_Config[AutoBarConfig.editPlayer].display = {};
	end

	local rows = AutoBarConfig:SliderGetValue("rows", calledFromSlider);
	local columns = AutoBarConfig:SliderGetValue("columns", calledFromSlider);
	local gapping = AutoBarConfig:SliderGetValue("gapping", calledFromSlider);
	local alpha = AutoBarConfig:SliderGetValue("alpha", calledFromSlider);
	local buttonWidth = AutoBarConfig:SliderGetValue("buttonWidth", calledFromSlider);
	local buttonHeight = AutoBarConfig:SliderGetValue("buttonHeight", calledFromSlider);
	local dockShiftX = AutoBarConfig:SliderGetValue("dockShiftX", calledFromSlider);
	local dockShiftY = AutoBarConfig:SliderGetValue("dockShiftY", calledFromSlider);

	if (calledFromSlider) then
		if (rows * columns > AUTOBAR_MAXBUTTONS) then
			if (this:GetName() == AutoBarConfigSliders["rows"]["name"]) then
				rows = getglobal(AutoBarConfigSliders["rows"]["name"]):GetValue();
				columns = math.floor(AUTOBAR_MAXBUTTONS / rows);
			else
				columns = getglobal(AutoBarConfigSliders["columns"]["name"]):GetValue();
				rows = math.floor(AUTOBAR_MAXBUTTONS / columns);
			end
		end
		if (buttonHeight ~= buttonWidth and not AutoBar_Config[AutoBarConfig.editPlayer].display.widthHeightUnlocked) then
			if (this:GetName() == AutoBarConfigSliders["buttonWidth"]["name"]) then
				buttonHeight = buttonWidth;
			elseif (this:GetName() == AutoBarConfigSliders["buttonHeight"]["name"]) then
				buttonWidth = buttonHeight;
			end
		end

		AutoBar_Config[AutoBarConfig.editPlayer].display.rows = rows;
		AutoBar_Config[AutoBarConfig.editPlayer].display.columns = columns;
		AutoBar_Config[AutoBarConfig.editPlayer].display.gapping = gapping;
		AutoBar_Config[AutoBarConfig.editPlayer].display.alpha = alpha;
		AutoBar_Config[AutoBarConfig.editPlayer].display.buttonWidth = buttonWidth;
		AutoBar_Config[AutoBarConfig.editPlayer].display.buttonHeight = buttonHeight;
		AutoBar_Config[AutoBarConfig.editPlayer].display.dockShiftX = dockShiftX;
		AutoBar_Config[AutoBarConfig.editPlayer].display.dockShiftY = dockShiftY;
	end

	AutoBarConfig:SliderSetValue("rows", rows);
	AutoBarConfig:SliderSetValue("columns", columns);
	AutoBarConfig:SliderSetValue("gapping", gapping);
	AutoBarConfig:SliderSetValue("alpha", alpha, math.floor(alpha) / 10);
	AutoBarConfig:SliderSetValue("buttonWidth", buttonWidth);
	AutoBarConfig:SliderSetValue("buttonHeight", buttonHeight);
	AutoBarConfig:SliderSetValue("dockShiftX", dockShiftX);
	AutoBarConfig:SliderSetValue("dockShiftY", dockShiftY);

	AutoBar_SetupVisual();
end


--
-- Checkbox handling
--

-- Disabling a layer disables editing of it as well
function AutoBarConfig:UseCheckBoxOnClick()
	local profile = AutoBar_Config[AutoBar.currentPlayer].profile;

	-- Only edit used layers
	if (not profile.useBasic and profile.edit == 4) then
		profile.edit = 0;
	end
	if (not profile.useClass and profile.edit == 3) then
		profile.edit = 0;
	end
	if (not profile.useShared and profile.edit == 2) then
		profile.edit = 0;
	end
	if (not profile.useCharacter and profile.edit == 1) then
		profile.edit = 0;
	end

	-- Assign editing to a used layer
	if (profile.edit == 0) then
		if (profile.useCharacter) then
			profile.edit = 1;
		elseif (profile.useShared) then
			profile.edit = 2;
		elseif (profile.useClass) then
			profile.edit = 3;
		elseif (profile.useBasic) then
			profile.edit = 4;
		end

		-- Enforce at least one used layer
		if (profile.edit == 0) then
			profile.useCharacter = true;
			profile.edit = 1;
		end
		AutoBarProfile:ProfileEditingChanged();
	end

	AutoBarProfile:ProfileChanged();
end


AutoBarConfig.checkboxes =
{
	["edit"] =
	{
		["variable"] = "profile",
		["name"] = "AutoBarConfigFrameSlotsEdit",
		["text1"] = AUTOBAR_CONFIG_EDITCHARACTER,
		["text2"] = AUTOBAR_CONFIG_EDITSHARED,
		["text3"] = AUTOBAR_CONFIG_EDITCLASS,
		["text4"] = AUTOBAR_CONFIG_EDITBASIC,
		["radio"] = 4,
		["default"] = 2,
		["callback"] = AutoBarProfile.ProfileEditingChanged,
	},
	["alignButtons"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameBarAlign",
		["text1"] = AUTOBAR_CONFIG_ALIGN,
		["radio"] = 3,
		["default"] = 1,
	},
	["hideDragHandle"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameBarHideDragHandle",
		["text"] = AUTOBAR_CONFIG_HIDEDRAGHANDLE,
	},
	["widthHeightUnlocked"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameBarWidthHeightUnlocked",
		["text"] = AUTOBAR_CONFIG_WIDTHHEIGHTUNLOCKED,
	},
	["hideKeyText"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameButtonsHideKeyText",
		["text"] = AUTOBAR_CONFIG_HIDEKEYBINDING,
	},
	["hideCount"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameButtonsHideCount",
		["text"] = AUTOBAR_CONFIG_HIDECOUNT,
	},
	["plainButtons"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameButtonsPlainButtons",
		["text"] = AUTOBAR_CONFIG_PLAINBUTTONS,
	},
	["autoSmartSelfCast"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameButtonsSmartSelfCast",
		["text"] = AUTOBAR_CONFIG_SMARTSELFCAST,
	},
	["showEmptyButtons"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameButtonsShowEmptyButtons",
		["text"] = AUTOBAR_CONFIG_SHOWEMPTY,
	},
	["showCategoryIcon"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameButtonsShowCategoryIcon",
		["text"] = AUTOBAR_CONFIG_SHOWCATEGORYICON,
	},
	["hideTooltips"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFrameButtonsHideTooltips",
		["text"] = AUTOBAR_CONFIG_HIDETOOLTIP,
	},
	["popupToTop"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFramePopupPopupToTop",
		["text"] = nil,
		["group"] = "PopupTo",
	},
	["popupToLeft"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFramePopupPopupToLeft",
		["text"] = nil,
		["group"] = "PopupTo",
	},
	["popupToRight"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFramePopupPopupToRight",
		["text"] = AUTOBAR_CONFIG_POPUPDIRECTION,
		["group"] = "PopupTo",
	},
	["popupToBottom"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFramePopupPopupToBottom",
		["text"] = nil,
		["group"] = "PopupTo",
	},
	["popupDisable"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFramePopupPopupDisable",
		["text"] = AUTOBAR_CONFIG_POPUPDISABLE,
	},
	["popupOnShift"] =
	{
		["variable"] = "display",
		["name"] = "AutoBarConfigFramePopupPopupOnShift",
		["text"] = AUTOBAR_CONFIG_POPUPONSHIFT,
	},
	["useCharacter"] =
	{
		["variable"] = "profile",
		["name"] = "AutoBarConfigFrameSlotsViewUseCharacter",
		["text"] = AUTOBAR_CONFIG_USECHARACTER,
		["callback"] = AutoBarConfig.UseCheckBoxOnClick,
	},
	["useShared"] =
	{
		["variable"] = "profile",
		["name"] = "AutoBarConfigFrameSlotsViewUseShared",
		["text"] = AUTOBAR_CONFIG_USESHARED,
		["callback"] = AutoBarConfig.UseCheckBoxOnClick,
	},
	["useClass"] =
	{
		["variable"] = "profile",
		["name"] = "AutoBarConfigFrameSlotsViewUseClass",
		["text"] = AUTOBAR_CONFIG_USECLASS,
		["callback"] = AutoBarConfig.UseCheckBoxOnClick,
	},
	["useBasic"] =
	{
		["variable"] = "profile",
		["name"] = "AutoBarConfigFrameSlotsViewUseBasic",
		["text"] = AUTOBAR_CONFIG_USEBASIC,
		["callback"] = AutoBarConfig.UseCheckBoxOnClick,
	},
	["layout"] =
	{
		["variable"] = "profile",
		["name"] = "AutoBarConfigFrameLayout",
		["text1"] = AUTOBAR_CONFIG_CHARACTERLAYOUT,
		["text2"] = AUTOBAR_CONFIG_SHAREDLAYOUT,
		["radio"] = 2,
		["default"] = 2,
		["callback"] = AutoBarProfile.ProfileChanged,
	},
	["hideConfigTooltips"] =
	{
		["variable"] = "profile",
		["name"] = "AutoBarConfigFrameProfileHideConfigTooltips",
		["text"] = AUTOBAR_CONFIG_HIDECONFIGTOOLTIPS,
	},
}


function AutoBarConfig:CheckBoxInitialize()
	local checkbox, checkboxText, text;
	for key, checkboxData in pairs(AutoBarConfig.checkboxes) do
		if (checkboxData["radio"]) then
			local x;
			if (checkboxData.variable == "profile") then
				x = AutoBar_Config[AutoBar.currentPlayer][checkboxData.variable][key];
			else
				x = AutoBar_Config[AutoBarConfig.editPlayer][checkboxData.variable][key];
			end
			if (not x) then
				x = checkboxData["default"] or 1;
			end

			for index = 1, checkboxData["radio"], 1 do
				checkbox = getglobal(checkboxData.name..index);
				checkboxText = getglobal(checkboxData.name .. index .. "Text");

				if (index ~= x) then
					checkbox:SetChecked(false);
				else
					checkbox:SetChecked(true);
				end

				text = checkboxData["text" .. index];
				if (text) then
					checkboxText:SetText(text);
				else
					checkboxText:Hide();
				end
			end
		else
			checkbox = getglobal(checkboxData.name);
			checkboxText = getglobal(checkboxData.name .. "Text");

			if (checkboxData.variable == "profile") then
				checkbox:SetChecked(AutoBar_Config[AutoBar.currentPlayer][checkboxData.variable][key]);
			else
				checkbox:SetChecked(AutoBar_Config[AutoBarConfig.editPlayer][checkboxData.variable][key]);
			end

			text = checkboxData["text"];
			if (text) then
				checkboxText:SetText(checkboxData["text"]);
			else
				checkboxText:Hide();
			end
		end
	end

	local profile = AutoBar_Config[AutoBar.currentPlayer].profile;
	if (profile.useBasic) then
		AutoBarConfigFrameSlotsEdit4:Show();
	else
		AutoBarConfigFrameSlotsEdit4:Hide();
	end
	if (profile.useClass) then
		AutoBarConfigFrameSlotsEdit3:Show();
	else
		AutoBarConfigFrameSlotsEdit3:Hide();
	end
	if (profile.useShared) then
		AutoBarConfigFrameSlotsEdit2:Show();
	else
		AutoBarConfigFrameSlotsEdit2:Hide();
	end
	if (profile.useCharacter) then
		AutoBarConfigFrameSlotsEdit1:Show();
	else
		AutoBarConfigFrameSlotsEdit1:Hide();
	end
end


function AutoBarConfig:CheckBoxOnClick()
	local buttonName = this:GetName();
	local group = nil;
	local theCheckboxData;
	for key, checkboxData in pairs(AutoBarConfig.checkboxes) do
		if (buttonName == checkboxData["name"]) then
			if (checkboxData.variable == "profile") then
				AutoBar_Config[AutoBar.currentPlayer][checkboxData.variable][key] = this:GetChecked();
			else
				AutoBar_Config[AutoBarConfig.editPlayer][checkboxData.variable][key] = this:GetChecked();
			end

			-- Radio button behavior
			group = checkboxData["group"];
			if (group) then
				-- Set clicked one to true
				AutoBar_Config[AutoBarConfig.editPlayer][checkboxData.variable][key] = true;
				this:SetChecked(true);

				-- Set other group members to false
				for keyGroup, checkboxDataGroup in pairs(AutoBarConfig.checkboxes) do
					if (buttonName ~= checkboxDataGroup.name and group == checkboxDataGroup.group) then
						AutoBar_Config[AutoBarConfig.editPlayer][checkboxData.variable][keyGroup] = false;
						getglobal(checkboxDataGroup.name):SetChecked(false);
					end
				end
			end
			theCheckboxData = checkboxData;
			break;
		-- This needs to be extended if there is a 10+ item radio group
		elseif (checkboxData["radio"] and string.sub(buttonName, 1, string.len(buttonName) - 1) == checkboxData["name"]) then
			this:SetChecked(true);
			local x = tonumber(string.sub(buttonName, string.len(buttonName)));
			if (checkboxData.variable == "profile") then
				AutoBar_Config[AutoBar.currentPlayer][checkboxData.variable][key] = x;
			else
				AutoBar_Config[AutoBarConfig.editPlayer][checkboxData.variable][key] = x;
			end
			for index = 1, checkboxData["radio"], 1 do
				if (index ~= x) then
					getglobal(checkboxData.name..x):SetChecked(false);
				end
			end
			theCheckboxData = checkboxData;
			break;
		end
	end

	if (theCheckboxData.callback) then
		theCheckboxData.callback();
	end

	AutoBarConfig:CheckBoxInitialize();
	AutoBar_SetupVisual();
end


--
-- Dropdown menu to choose the profile we are editing
--

-- Using _SHARED so it can be treated like AutoBar.currentPlayer with no chance of colliding with a real player name.
AutoBarConfig.sharedProfiles = {
		["_SHARED1"] = AUTOBAR_CONFIG_SHARED1,
		["_SHARED2"] = AUTOBAR_CONFIG_SHARED2,
		["_SHARED3"] = AUTOBAR_CONFIG_SHARED3,
		["_SHARED4"] = AUTOBAR_CONFIG_SHARED4,
	};

function AutoBarConfig:SetSharedInitialize()
	if (not AutoBarConfigFrameProfileSetShared.dewdrop) then
		AutoBarConfigFrameProfileSetSharedButton.dewdrop = dewdrop;
		dewdrop:Register(AutoBarConfigFrameProfileSetShared, 'children', AutoBarConfig.SetSharedSetup,
			'point', function(parent) return "TOPLEFT", "TOPRIGHT" end
		);
	end
	local setSharedTarget = AutoBarConfig:GetSetSharedTarget();
	AutoBarConfigFrameProfileSetSharedText:SetText(AutoBarConfig.sharedProfiles[setSharedTarget]);
end


function AutoBarConfig:GetSetSharedTarget()
	local setSharedTarget = "_SHARED1";
	if (AutoBar_Config and AutoBar_Config[AutoBar.currentPlayer] and AutoBar_Config[AutoBar.currentPlayer].profile and AutoBar_Config[AutoBar.currentPlayer].profile.shared) then
		setSharedTarget = AutoBar_Config[AutoBar.currentPlayer].profile.shared;
	end

	-- Get rid of obsolete shared names
	if (not AutoBarConfig.sharedProfiles[setSharedTarget]) then
		setSharedTarget = "_SHARED1";
	end

	return setSharedTarget;
end


function AutoBarConfig.SetSharedSetup()
	local checked;
	local setSharedTarget = AutoBarConfig:GetSetSharedTarget();
	for targetName, targetText in pairs(AutoBarConfig.sharedProfiles) do
		if (targetName == setSharedTarget) then
			checked = true;
		    AutoBarConfigFrameProfileSetSharedText:SetText(targetText);
		else
			checked = false;
		end
		dewdrop:AddLine(
			'text', targetText,
			'checked', checked,
			'func', AutoBarConfig.SetSharedOnClick,
			'arg1', targetName,
			'arg2', targetText,
			'closeWhenClicked', true
		);
	end
end


-- SetShared Dropdown Menu callback
function AutoBarConfig.SetSharedOnClick(targetName, targetText)
	AutoBar_Config[AutoBar.currentPlayer].profile.shared = targetName;

	AutoBarProfile:ProfileChanged();
	AutoBarConfig.OnShow();
    AutoBarConfigFrameProfileSetSharedText:SetText(targetText);
	AutoBar_SetupVisual();
end


--
-- Dropdown menu for docking to various frames
--

AutoBarConfig.dockingFrames = {
		["NONE"] = {
			text = AUTOBAR_CONFIG_DOCKTONONE,
			offset = { x = 0, y = 0, point = "BOTTOMLEFT", relative = "TOPLEFT" },
		},
		["GSecondBar"] = {
			text = AUTOBAR_CONFIG_DOCKTOGBARS,
			offset = { x = 0, y = 0, point = "BOTTOMLEFT", relative = "TOPLEFT" },
		},
		["MainMenuBarArtFrame"] = {
			text = AUTOBAR_CONFIG_DOCKTOMAIN,
			offset = { x = 0, y = 0, point = "BOTTOMRIGHT", relative = "TOPRIGHT" },
		},
		["ChatFrame1"] = {
			text = AUTOBAR_CONFIG_DOCKTOCHATFRAME,
			offset = { x = 0, y = 25, point = "BOTTOMLEFT", relative = "TOPLEFT" },
		},
		["ChatFrameMenuButton"] = {
			text = AUTOBAR_CONFIG_DOCKTOCHATFRAMEMENU,
			offset = { x = 0, y = 25, point = "BOTTOMLEFT", relative = "TOPLEFT" },
		},
		["MainMenuBar"] = {
			text = AUTOBAR_CONFIG_DOCKTOACTIONBAR,
			offset = { x = 7, y = 40, point = "BOTTOMLEFT", relative = "TOPLEFT" },
		},
		["CharacterMicroButton"] = {
			text = AUTOBAR_CONFIG_DOCKTOMENUBUTTONS,
			offset = { x = 0, y = 0, point = "BOTTOMLEFT", relative = "BOTTOMLEFT" },
		},
	};

function AutoBarConfig:DockToInitialize()
	if (not AutoBarConfigFrameBarDockTo.dewdrop) then
		AutoBarConfigFrameBarDockToButton.dewdrop = dewdrop;
		dewdrop:Register(AutoBarConfigFrameBarDockTo, 'children', AutoBarConfig.DockToSetup,
			'point', function(parent) return "TOPLEFT", "TOPRIGHT" end
		);
	end
	local dockToFrame = AutoBarConfig:GetDockToFrame();
	AutoBarConfigFrameBarDockToText:SetText(AutoBarConfig.dockingFrames[dockToFrame].text);
end


function AutoBarConfig:GetDockToFrame()
	local dockToFrame = "NONE";
	if (AutoBar_Config and AutoBar_Config[AutoBarConfig.editPlayer] and AutoBar_Config[AutoBarConfig.editPlayer].display and AutoBar_Config[AutoBarConfig.editPlayer].display.docking) then
		dockToFrame = AutoBar_Config[AutoBarConfig.editPlayer].display.docking;
	end
	return dockToFrame;
end


function AutoBarConfig.DockToSetup()
	local checked, frameText;
	local dockToFrame = AutoBarConfig:GetDockToFrame();
	for frameName, frameInfo in pairs(AutoBarConfig.dockingFrames) do
--DEFAULT_CHAT_FRAME:AddMessage("AutoBarConfig.DockToSetup " ..frameName.. " " ..dockToFrame);
		frameText = frameInfo.text;
		if (frameName == dockToFrame) then
			checked = true;
		    AutoBarConfigFrameBarDockToText:SetText(frameText);
		else
			checked = false;
		end
		dewdrop:AddLine(
			'text', frameText,
			'checked', checked,
			'func', AutoBarConfig.DockToOnClick,
			'arg1', frameName,
			'arg2', frameText,
			'closeWhenClicked', true
		);
	end
end


-- DockTo Dropdown Menu callback
function AutoBarConfig.DockToOnClick(frameName, frameText)
	if (frameName ~= "NONE") then
		AutoBar_Config[AutoBarConfig.editPlayer].display.docking = frameName;
	else
		AutoBar_Config[AutoBarConfig.editPlayer].display.docking = nil;
	end
    AutoBarConfigFrameBarDockToText:SetText(frameText);
	AutoBar_SetupVisual();
end


--
-- Profile Quick Setup Buttons
--

function AutoBarConfig:SetupSingle()
	AutoBarProfile:SetDefaultsSingle();
	AutoBarProfile:ProfileEditingChanged();
end


function AutoBarConfig:SetupShared()
	AutoBarProfile:SetDefaultsShared();
	AutoBarProfile:ProfileEditingChanged();
end


function AutoBarConfig:SetupStandard()
	AutoBarProfile:SetDefaultsStandard();
	AutoBarProfile:ProfileEditingChanged();
end


function AutoBarConfig:SetupBlankSlate()
	AutoBarProfile:SetDefaultsBlankSlate();
	AutoBarProfile:ProfileEditingChanged();
end


function AutoBarConfig:ResetSingle()
	AutoBarProfile:SetDefaultsSingle();
	AutoBarProfile:ButtonsCopySingle();
	AutoBarProfile:ProfileEditingChanged();
end


function AutoBarConfig:ResetShared()
	AutoBarProfile:SetDefaultsShared();
	AutoBarProfile:ButtonsCopyShared();
	AutoBarProfile:ProfileEditingChanged();
end


function AutoBarConfig:ResetStandard()
	AutoBarProfile:SetDefaultsStandard();
	AutoBar_Config[AutoBar.currentPlayer].buttons = nil;
	AutoBar_Config[AutoBar_Config[AutoBar.currentPlayer].profile.shared].buttons = nil;
	if (AutoBar_Config[AutoBarProfile.CLASSPROFILE]) then
		AutoBar_Config[AutoBarProfile.CLASSPROFILE].buttons = nil;
	end
	if (AutoBar_Config["_BASIC"]) then
		AutoBar_Config["_BASIC"].buttons = nil;
	end
	AutoBarProfile:ProfileEditingChanged();
	AutoBarProfile.Initialize();
	AutoBarProfile:ButtonsChanged()
	AutoBarConfig.OnShow();
end


--
-- Tab panels
--

local tabFrameNames = { "Slots", "Bar", "Buttons", "Popup", "Profile" };

function AutoBarConfig:TabButtonOnClick(tabId)
	PanelTemplates_SetTab(AutoBarConfigFrame, tabId);
	PanelTemplates_UpdateTabs(AutoBarConfigFrame);
	AutoBar_Config[AutoBar.currentPlayer].display.selectedTab = tabId;
	for index, tabFrameName in pairs(tabFrameNames) do
		if (tabFrameName == tabFrameNames[tabId]) then
			getglobal("AutoBarConfigFrame" .. tabFrameName):Show();
		else
			getglobal("AutoBarConfigFrame" .. tabFrameName):Hide();
		end
	end

	if (tabFrameNames[tabId] == "Slots" or tabFrameNames[tabId] == "Profile") then
		AutoBarConfigFrameSlotsView:Show();
	else
		AutoBarConfigFrameSlotsView:Hide();
	end

	if (tabFrameNames[tabId] == "Slots") then
		AutoBarConfigFrameLayout1:Hide();
		AutoBarConfigFrameLayout2:Hide();
	else
		AutoBarConfigFrameLayout1:Show();
		AutoBarConfigFrameLayout2:Show();
	end
end


-- Initialize the config tabs
function AutoBarConfig:TabButtonInitialize()
	PanelTemplates_SetNumTabs(AutoBarConfigFrame, table.getn(tabFrameNames));
	if (not AutoBar_Config[AutoBar.currentPlayer].display.selectedTab) then
		AutoBar_Config[AutoBar.currentPlayer].display.selectedTab = 1;
	end

	AutoBarConfig:TabButtonOnClick(AutoBar_Config[AutoBar.currentPlayer].display.selectedTab)
	PanelTemplates_UpdateTabs(AutoBarConfigFrame);
end


-- Reset the the current tab to default
function AutoBarConfig:TabReset()
	local tabFrameName = tabFrameNames[AutoBar_Config[AutoBar.currentPlayer].display.selectedTab];
	if (tabFrameName == "Slots") then
		AutoBarConfig.SlotsReset();
	elseif (tabFrameName == "Profile") then
		AutoBarConfig.ProfileReset();
	else
		AutoBarConfig.DisplayReset();
	end
end


-- Reset the buttons to default
function AutoBarConfig.SlotsReset()
	AutoBar_Config[AutoBarConfig.editPlayer].buttons = nil;
	AutoBarProfile.Initialize();
	AutoBarProfile:ButtonsChanged()
	AutoBarConfig.OnShow();
end


-- Reset the display to default
function AutoBarConfig.DisplayReset()
	AutoBar_Config[AutoBarConfig.editPlayer].display = {};
	AutoBarProfile.Initialize();
	AutoBarProfile:DisplayChanged();
	AutoBarConfig.OnShow();
end


-- Reset the profile to default
function AutoBarConfig.ProfileReset()
	AutoBarProfile:SetDefaults();
	AutoBarProfile.Initialize();
	AutoBarProfile:ProfileChanged();
	AutoBarConfig.OnShow();
end

--DEFAULT_CHAT_FRAME:AddMessage("AutoBar.smartSelfcast category " .. category);
